@@ -29,12 +29,13 @@ import pyatspi
 from gi.repository import Atspi, Atk
 
 from . import braille
+from . import debug
 from . import generator
+from . import messages
 from . import orca_state
 from . import settings
 from . import settings_manager
 from .braille_rolenames import shortRoleNames
-from .orca_i18n import ngettext
 
 _settingsManager = settings_manager.getManager()
 
@@ -72,6 +73,11 @@ class BrailleGenerator(generator.Generator):
         globalsDict['asString'] = self.asString
 
     def generateBraille(self, obj, **args):
+        if not _settingsManager.getSetting('enableBraille') \
+           and not _settingsManager.getSetting('enableBrailleMonitor'):
+            debug.println(debug.LEVEL_INFO, "BRAILLE: generation disabled")
+            return [[], None]
+
         if obj == orca_state.locusOfFocus \
            and not args.get('formatType', None):
             args['formatType'] = 'focused'
@@ -132,7 +138,7 @@ class BrailleGenerator(generator.Generator):
 
         # egg-list-box, e.g. privacy panel in gnome-control-center
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
-            doNotPresent.append(pyatspi.ROLE_FILLER)
+            doNotPresent.append(obj.getRole())
 
         if verbosityLevel == settings.VERBOSITY_LEVEL_BRIEF:
             doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
@@ -175,6 +181,15 @@ class BrailleGenerator(generator.Generator):
 
         return Atk.role_get_localized_name(atkRole)
 
+    def _generateUnrelatedLabels(self, obj, **args):
+        result = []
+        labels = self._script.utilities.unrelatedLabels(obj)
+        for label in labels:
+            name = self._generateName(label, **args)
+            result.extend(name)
+
+        return result
+
     #####################################################################
     #                                                                   #
     # Keyboard shortcut information                                     #
@@ -211,13 +226,8 @@ class BrailleGenerator(generator.Generator):
         except:
             alertAndDialogCount = 0
         if alertAndDialogCount > 0:
-            # Translators: this tells the user how many unfocused
-            # alert and dialog windows plus the total number of
-            # windows that this application has.
-            #
-            result.append(ngettext("(%d dialog)",
-                                   "(%d dialogs)",
-                                   alertAndDialogCount) % alertAndDialogCount)
+             result.append(messages.dialogCountBraille(alertAndDialogCount))
+
         return result
 
     def _generateAncestors(self, obj, **args):
