@@ -24,34 +24,30 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2011. Orca Team."
 __license__   = "LGPL"
 
-from gi.repository import GObject
+from gi.repository import GLib
 import pyatspi
-import Queue
+import queue
 import threading
 import time
 
-import braille
-import debug
-import input_event
-import orca
-import orca_state
-import settings
-import speech
+from . import debug
+from . import input_event
+from . import messages
+from . import orca_state
+from . import script_manager
+from . import settings
 
-from orca_i18n import _
-
-_scriptManager = None
+_scriptManager = script_manager.getManager()
 
 class EventManager:
 
     def __init__(self):
-
         debug.println(debug.LEVEL_FINEST, 'INFO: Initializing event manager')
         self._scriptListenerCounts = {}
         self.registry = pyatspi.Registry
         self._enqueueCount = 0
         self._dequeueCount = 0
-        self._eventQueue     = Queue.Queue(0)
+        self._eventQueue     = queue.Queue(0)
         self._gidleId        = 0
         self._gidleLock      = threading.Lock()
         self.noFocusTimestamp = 0.0
@@ -61,14 +57,6 @@ class EventManager:
         """Called when this presentation manager is activated."""
 
         debug.println(debug.LEVEL_FINEST, 'INFO: Activating event manager')
-
-        global _scriptManager
-        _scriptManager = getattr(orca, '_scriptManager')
-
-        # Tell BrlTTY which commands we care about.
-        #
-        braille.setupKeyRanges(orca_state.activeScript.brailleBindings.keys())
-
         self._registerListener("window:activate")
         self._registerListener("window:deactivate")
         self._registerListener("object:children-changed")
@@ -79,7 +67,7 @@ class EventManager:
         """Called when this event manager is deactivated."""
 
         debug.println(debug.LEVEL_FINEST, 'INFO: Dectivating event manager')
-        for eventType in self._scriptListenerCounts.keys():
+        for eventType in list(self._scriptListenerCounts.keys()):
             self.registry.deregisterEventListener(self._enqueue, eventType)
         self._scriptListenerCounts = {}
         debug.println(debug.LEVEL_FINEST, 'INFO: Event manager deactivated')
@@ -87,13 +75,10 @@ class EventManager:
     def _ignore(self, event):
         """Returns True if this event should be ignored."""
 
-        if not event or not event.source:
-            return True
-
         ignoredList = ['object:state-changed:defunct',
                        'object:property-change:accessible-parent']
         ignoredList.extend(settings.ignoredEventsList)
-        if filter(event.type.startswith, ignoredList):
+        if list(filter(event.type.startswith, ignoredList)):
             return True
 
         # This should ultimately be changed as there are valid reasons
@@ -123,7 +108,7 @@ class EventManager:
         if asyncMode and not self._gidleId:
             if settings.gilSleepTime:
                 time.sleep(settings.gilSleepTime)
-            self._gidleId = GObject.idle_add(self._dequeue)
+            self._gidleId = GLib.idle_add(self._dequeue)
 
         if debugging:
             debug.println(debug.LEVEL_ALL, "           releasing lock...")
@@ -234,32 +219,27 @@ class EventManager:
             # some reason if done inside an acquire/release block for a
             # lock.  So...we do it here.]]]
             #
-            noFocus = \
-                not orca_state.activeScript \
-                or (not orca_state.locusOfFocus \
-                    and self.noFocusTimestamp != orca_state.noFocusTimestamp)
+            try:
+                noFocus = \
+                    not orca_state.activeScript \
+                    or (not orca_state.locusOfFocus and \
+                        self.noFocusTimestamp != orca_state.noFocusTimestamp)
+            except:
+                noFocus = True
 
             self._gidleLock.acquire()
             if self._eventQueue.empty():
                 if noFocus:
                     if settings.gilSleepTime:
                         time.sleep(settings.gilSleepTime)
-                    # Translators: this is intended to be a short phrase to
-                    # speak and braille to tell the user that no component
-                    # has keyboard focus.
-                    #
-                    message = _("No focus")
-                    if settings.brailleVerbosityLevel == \
-                            settings.VERBOSITY_LEVEL_VERBOSE:
-                        braille.displayMessage(message)
-                    if settings.speechVerbosityLevel == \
-                            settings.VERBOSITY_LEVEL_VERBOSE:
-                        speech.speak(message)
+                    fullMessage = messages.NO_FOCUS
+                    defaultScript = _scriptManager.getDefaultScript()
+                    defaultScript.presentMessage(fullMessage, '')
                     self.noFocusTimestamp = orca_state.noFocusTimestamp
                 self._gidleId = 0
                 rerun = False # destroy and don't call again
             self._gidleLock.release()
-        except Queue.Empty:
+        except queue.Empty:
             debug.println(debug.LEVEL_SEVERE,
                           "event_manager._dequeue: the event queue is empty!")
             self._gidleId = 0
@@ -322,7 +302,7 @@ class EventManager:
                       'INFO: Event manager registering listeners for: %s' \
                        % script)
 
-        for eventType in script.listeners.keys():
+        for eventType in list(script.listeners.keys()):
             self._registerListener(eventType)
 
     def deregisterScriptListeners(self, script):
@@ -337,19 +317,19 @@ class EventManager:
                       'INFO: Event manager deregistering listeners for: %s' \
                        % script)
 
-        for eventType in script.listeners.keys():
+        for eventType in list(script.listeners.keys()):
             self._deregisterListener(eventType)
 
     def registerModuleListeners(self, listeners):
         """Register the listeners on behalf of the caller."""
 
-        for eventType, function in listeners.items():
+        for eventType, function in list(listeners.items()):
             self.registry.registerEventListener(function, eventType)
 
     def deregisterModuleListeners(self, listeners):
         """Deegister the listeners on behalf of the caller."""
 
-        for eventType, function in listeners.items():
+        for eventType, function in list(listeners.items()):
             self.registry.deregisterEventListener(function, eventType)
 
     def registerKeystrokeListener(self, function, mask=None, kind=None):
@@ -361,7 +341,7 @@ class EventManager:
              % function)
 
         if mask == None:
-            mask = range(256)
+            mask = list(range(256))
 
         if kind == None:
             kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
@@ -377,7 +357,7 @@ class EventManager:
              % function)
 
         if mask == None:
-            mask = range(256)
+            mask = list(range(256))
 
         if kind == None:
             kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
@@ -426,8 +406,13 @@ class EventManager:
         script = None
         try:
             app = event.host_application or event.source.getApplication()
+            if app and app.getState().contains(pyatspi.STATE_DEFUNCT):
+                msg = 'WARNING: App is defunct. Cannot get script for event.'
+                debug.println(debug.LEVEL_WARNING, msg)
+                return None
         except:
-            debug.printException(debug.LEVEL_WARNING)
+            msg = 'WARNING: Exception when getting script for event.'
+            debug.println(debug.LEVEL_WARNING, msg)
         else:
             script = _scriptManager.getScript(app, event.source)
 
@@ -499,10 +484,6 @@ class EventManager:
             try:
                 if event.source == self.registry.getDesktop(0):
                     _scriptManager.reclaimScripts()
-                    if settings.debugMemoryUsage:
-                        orca.cleanupGarbage()
-                    if not event.source.childCount:
-                        orca.shutdown()
                     return
             except (LookupError, RuntimeError):
                 # If we got this error here, we'll get it again when we
@@ -519,7 +500,6 @@ class EventManager:
            and orca_state.activeScript \
            and orca_state.activeScript.flatReviewContext \
            and orca_state.activeScript.app == event.host_application:
-            orca_state.activeScript.drawOutline(-1, 0, 0, 0)
             orca_state.activeScript.flatReviewContext = None
 
         try:
@@ -528,17 +508,16 @@ class EventManager:
             debug.println(debug.LEVEL_WARNING,
                           "Error while processing event: %s" % eType)
             if eType.startswith("window:deactivate"):
-                orca.setLocusOfFocus(event, None)
+                orca_state.locusOfFocus = None
                 orca_state.activeWindow = None
             return
         except:
-            debug.printException(debug.LEVEL_WARNING)
             return
 
         if state and state.contains(pyatspi.STATE_DEFUNCT):
             debug.println(debug.LEVEL_FINEST, "IGNORING DEFUNCT OBJECT")
             if eType.startswith("window:deactivate"):
-                orca.setLocusOfFocus(event, None)
+                orca_state.locusOfFocus = None
                 orca_state.activeWindow = None
             return
 
@@ -551,6 +530,9 @@ class EventManager:
             debug.printDetails(debug.LEVEL_FINEST, "    ", event.source)
 
         script = self._getScriptForEvent(event)
+        if not script:
+            return
+
         debug.println(debug.LEVEL_FINEST, "Script for event: %s" % script.name)
         setNewActiveScript, reason = self._isActivatableEvent(event, script)
         if setNewActiveScript:
@@ -595,3 +577,8 @@ class EventManager:
             return True
         else:
             return False
+
+_manager = EventManager()
+
+def getManager():
+    return _manager
