@@ -31,13 +31,13 @@ import re
 from . import braille
 from . import debug
 from . import eventsynthesizer
+from . import messages
+from . import object_properties
 from . import orca_state
 from . import settings
 
 from .braille_generator import BrailleGenerator
 from .orca_i18n import _
-from .orca_i18n import C_
-from .orca_i18n import ngettext
 
 # [[[WDW - HACK Regular expression to split strings on whitespace
 # boundaries, which is what we'll use for word dividers instead of
@@ -350,43 +350,27 @@ class StateZone(Zone):
                              pyatspi.ROLE_CHECK_MENU_ITEM,
                              pyatspi.ROLE_TABLE_CELL]:
                 if stateCount == 2:
-                    # Translators: this represents the state of a checkbox.
-                    #
-                    speechState = _("partially checked")
+                    speechState = object_properties.STATE_PARTIALLY_CHECKED
                 elif stateCount == 1:
-                    # Translators: this represents the state of a checkbox.
-                    #
-                    speechState = _("checked")
+                    speechState = object_properties.STATE_CHECKED
                 else:
-                    # Translators: this represents the state of a checkbox.
-                    #
-                    speechState = _("not checked")
+                    speechState = object_properties.STATE_NOT_CHECKED
                 brailleState = \
-                    settings.brailleCheckBoxIndicators[stateCount]
+                    object_properties.CHECK_BOX_INDICATORS_BRAILLE[stateCount]
             elif self.role == pyatspi.ROLE_TOGGLE_BUTTON:
                 if stateCount:
-                    # Translators: the state of a toggle button.
-                    #
-                    speechState = _("pressed")
+                    speechState = object_properties.STATE_PRESSED
                 else:
-                    # Translators: the state of a toggle button.
-                    #
-                    speechState = _("not pressed")
+                    speechState = object_properties.STATE_NOT_PRESSED
                 brailleState = \
-                    settings.brailleRadioButtonIndicators[stateCount]
+                    object_properties.RADIO_BUTTON_INDICATORS_BRAILLE[stateCount]
             else:
                 if stateCount:
-                    # Translators: this is in reference to a radio button being
-                    # selected or not.
-                    #
-                    speechState = C_("radiobutton", "selected")
+                    speechState = object_properties.STATE_SELECTED_RADIO_BUTTON
                 else:
-                    # Translators: this is in reference to a radio button being
-                    # selected or not.
-                    #
-                    speechState = C_("radiobutton", "not selected")
+                    speechState = object_properties.STATE_UNSELECTED_RADIO_BUTTON
                 brailleState = \
-                    settings.brailleRadioButtonIndicators[stateCount]
+                    object_properties.RADIO_BUTTON_INDICATORS_BRAILLE[stateCount]
 
             if attr == "string":
                 return speechState
@@ -420,13 +404,9 @@ class ValueZone(Zone):
                              pyatspi.ROLE_SCROLL_BAR]:
                 stateset = self.accessible.getState()
                 if stateset.contains(pyatspi.STATE_HORIZONTAL):
-                    # Translators: The component orientation is horizontal.
-                    #
-                    orientation = _("horizontal")
+                    orientation = object_properties.STATE_HORIZONTAL
                 elif stateset.contains(pyatspi.STATE_VERTICAL):
-                    # Translators: The component orientation is vertical.
-                    #
-                    orientation = _("vertical")
+                    orientation = object_properties.STATE_VERTICAL
                         
             try:
                 value = self.accessible.queryValue()
@@ -445,15 +425,8 @@ class ValueZone(Zone):
                 speechValue = orientation + " " + rolename
             else:
                 speechValue = rolename
-                
-            # Translators: this is the percentage value of a slider, 
-            # progress bar or other component that displays a value as 
-            # a percentage.
-            #
-            percentString = ngettext("%d percent.",
-                                     "%d percent.",
-                                     percentValue) % percentValue
-            speechValue = speechValue + " " + percentString
+
+            speechValue = speechValue + " " + messages.percentage(percentValue)
 
             rolename = BrailleGenerator.getLocalizedRoleName(self.accessible)
             if orientation:
@@ -874,7 +847,7 @@ class Context:
         offset = 0
         lastEndOffset = -1
         upperMax = lowerMax = text.characterCount
-        upperMid = lowerMid = upperMax /2
+        upperMid = lowerMid = int(upperMax / 2)
         upperMin = lowerMin = 0
         upperY = lowerY = 0
         oldMid = 0
@@ -890,7 +863,7 @@ class Context:
                 upperMax = upperMid
             else:
                 upperMin = upperMid
-            upperMid = (upperMax - upperMin) /2 + upperMin
+            upperMid = int((upperMax - upperMin) / 2) + upperMin
 
         # performing binary search to locate last line inside clipped area
         oldMid = 0
@@ -905,7 +878,7 @@ class Context:
                 lowerMax = lowerMid
             else:
                 lowerMin = lowerMid
-            lowerMid = (lowerMax - lowerMin) /2 + lowerMin
+            lowerMid = int((lowerMax - lowerMin) / 2) + lowerMin
 
         # finding out the zones
         offset = upperMin
@@ -1219,9 +1192,12 @@ class Context:
             elif accessible.description and len(accessible.description):
                 string = accessible.description
 
+            if not string and role == pyatspi.ROLE_ICON:
+                string = self.script.utilities.displayedText(accessible)
+
             if (string == "") \
                 and (role != pyatspi.ROLE_TABLE_CELL):
-                string = accessible.getRoleName()
+                string = accessible.getLocalizedRoleName()
 
             if len(string) and ((clipping[2] != 0) or (clipping[3] != 0)):
                 zones.append(Zone(accessible,
@@ -1419,56 +1395,6 @@ class Context:
 
         return lines
 
-    def _dumpCurrentState(self):
-        print("line=%d, zone=%d, word=%d, char=%d" \
-              % (self.lineIndex,
-                 self.zoneIndex,
-                 self.wordIndex,
-                 self.zoneIndex))
-
-        zone = self.lines[self.lineIndex].zones[self.zoneIndex]
-        text = zone.accessible.queryText()
-
-        if not text:
-            print("  Not Accessibility_Text")
-            return
-
-        print("  getTextBeforeOffset: %d" % text.caretOffset)
-        [string, startOffset, endOffset] = text.getTextBeforeOffset(
-            text.caretOffset,
-            pyatspi.TEXT_BOUNDARY_WORD_START)
-        print("    WORD_START: start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string))
-        [string, startOffset, endOffset] = text.getTextBeforeOffset(
-            text.caretOffset,
-            pyatspi.TEXT_BOUNDARY_WORD_END)
-        print("    WORD_END:   start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string))
-
-        print("  getTextAtOffset: %d" % text.caretOffset)
-        [string, startOffset, endOffset] = text.getTextAtOffset(
-            text.caretOffset,
-            pyatspi.TEXT_BOUNDARY_WORD_START)
-        print("    WORD_START: start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string))
-        [string, startOffset, endOffset] = text.getTextAtOffset(
-            text.caretOffset,
-            pyatspi.TEXT_BOUNDARY_WORD_END)
-        print("    WORD_END:   start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string))
-
-        print("  getTextAfterOffset: %d" % text.caretOffset)
-        [string, startOffset, endOffset] = text.getTextAfterOffset(
-            text.caretOffset,
-            pyatspi.TEXT_BOUNDARY_WORD_START)
-        print("    WORD_START: start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string))
-        [string, startOffset, endOffset] = text.getTextAfterOffset(
-            text.caretOffset,
-            pyatspi.TEXT_BOUNDARY_WORD_END)
-        print("    WORD_END:   start=%d end=%d string='%s'" \
-              % (startOffset, endOffset, string))
-
     def setCurrent(self, lineIndex, zoneIndex, wordIndex, charIndex):
         """Sets the current character of interest.
 
@@ -1800,6 +1726,12 @@ class Context:
                     or (len(zone.words) \
                         and zone.words[self.wordIndex].string.isspace())):
 
+                hasMoreText = False
+                if self.lineIndex > 0 and isinstance(zone, TextZone):
+                    prevZone = self.lines[self.lineIndex - 1].zones[-1]
+                    if prevZone.accessible == zone.accessible:
+                        hasMoreText = True
+
                 # If we're on whitespace in the same zone, then let's
                 # try to move on.  If not, we've definitely moved
                 # across accessibles.  If that's the case, let's try
@@ -1808,7 +1740,7 @@ class Context:
                 # with no words and we should do our best to announce
                 # this to the user (e.g., "whitespace" or "blank").
                 #
-                if zone.accessible == accessible:
+                if zone.accessible == accessible or hasMoreText:
                     moved = self.goPrevious(Context.WORD, wrap)
                 else:
                     wordIndex = self.wordIndex - 1
