@@ -31,11 +31,13 @@ import traceback
 
 import pyatspi
 
-import braille
-import debug
-import settings
+from . import braille
+from . import debug
+from . import messages
+from . import settings
 
-from orca_i18n import _         # for gettext support
+from .orca_i18n import _         # for gettext support
+import collections
 
 def _formatExceptionInfo(maxTBlevel=5):
     cla, exc, trbk = sys.exc_info()
@@ -71,10 +73,7 @@ class Generator:
         self._script = script
         self._methodsDict = {}
         for method in \
-            filter(lambda z: callable(z),
-                   map(lambda y: getattr(self, y).__get__(self, self.__class__),
-                       filter(lambda x: x.startswith(METHOD_PREFIX),
-                                        dir(self)))):
+            [z for z in [getattr(self, y).__get__(self, self.__class__) for y in [x for x in dir(self) if x.startswith(METHOD_PREFIX)]] if isinstance(z, collections.Callable)]:
             name = method.__name__[len(METHOD_PREFIX):]
             name = name[0].lower() + name[1:]
             self._methodsDict[name] = method
@@ -97,7 +96,7 @@ class Generator:
         # for the results of all the legal method names.
         #
         globalsDict = {}
-        for key in self._methodsDict.keys():
+        for key in list(self._methodsDict.keys()):
             globalsDict[key] = []
         self._addGlobals(globalsDict)
 
@@ -122,7 +121,7 @@ class Generator:
                             arg = info[1][0]
                             arg = arg.replace("name '", "")
                             arg = arg.replace("' is not defined", "")
-                            if not self._methodsDict.has_key(arg):
+                            if arg not in self._methodsDict:
                                 debug.printException(debug.LEVEL_SEVERE)
                                 debug.println(
                                     debug.LEVEL_SEVERE,
@@ -248,7 +247,7 @@ class Generator:
                     arg = info[1][0]
                     arg = arg.replace("name '", "")
                     arg = arg.replace("' is not defined", "")
-                    if not self._methodsDict.has_key(arg):
+                    if arg not in self._methodsDict:
                         debug.printException(debug.LEVEL_SEVERE)
                         debug.println(
                             debug.LEVEL_SEVERE,
@@ -330,10 +329,8 @@ class Generator:
         the assumption being that the user was able to see the text prior
         to giving the widget focus.
         """
-        result = filter(lambda x:
-                        x.startswith('placeholder-text:'),
-                        obj.getAttributes())
-        return map(lambda x: x.replace('placeholder-text:', ''), result)
+        result = [x for x in obj.getAttributes() if x.startswith('placeholder-text:')]
+        return [x.replace('placeholder-text:', '') for x in result]
 
     def _generateLabelAndName(self, obj, **args):
         """Returns the label and the name as an array of strings for speech
@@ -520,6 +517,23 @@ class Generator:
             result.append(indicators[0])
         return result
 
+    def _generateChildWidget(self, obj, **args):
+        widgetRoles = [pyatspi.ROLE_CHECK_BOX,
+                       pyatspi.ROLE_COMBO_BOX,
+                       pyatspi.ROLE_PUSH_BUTTON,
+                       pyatspi.ROLE_RADIO_BUTTON,
+                       pyatspi.ROLE_SLIDER,
+                       pyatspi.ROLE_TOGGLE_BUTTON]
+        isWidget = lambda x: x and x.getRole() in widgetRoles
+
+        # For GtkListBox, such as those found in the control center
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            widget = pyatspi.findDescendant(obj, isWidget)
+            if widget:
+                return self.generate(widget, includeContext=False)
+
+        return []
+
     def _generateToggleState(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the checked state of the object.  This is typically
@@ -832,6 +846,16 @@ class Generator:
         self._restoreRole(oldRole, args)
         return result
 
+    def _generateTable(self, obj, **args):
+        """Returns an array of strings for use by speech and braille to present
+        the size of a table."""
+
+        try:
+            table = obj.queryTable()
+        except:
+            return []
+        return [messages.tableSize(table.nRows, table.nColumns)]       
+
     def _generateTableCellRow(self, obj, **args):
         """Orca has a feature to automatically read an entire row of a table
         as the user arrows up/down the roles.  This leads to complexity in
@@ -875,8 +899,13 @@ class Generator:
                          and pointOfReference["lastColumn"] == column))
             if presentAll:
                 args['readingRow'] = True
-                for i in range(0, parentTable.nColumns):
-                    cell = parentTable.getAccessibleAt(row, i)
+                if self._script.utilities.isTableRow(obj):
+                    cells = [x for x in obj]
+                else:
+                    cells = [parentTable.getAccessibleAt(row, i) \
+                                 for i in range(parentTable.nColumns)]
+
+                for cell in cells:
                     if not cell:
                         continue
                     state = cell.getState()
@@ -916,11 +945,6 @@ class Generator:
         if not displayedText:
             return []
 
-        try:
-            displayedText = displayedText.decode('UTF-8')
-        except (UnicodeDecodeError, UnicodeEncodeError):
-            pass
-
         return [displayedText]
 
     #####################################################################
