@@ -39,6 +39,8 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import imp
+import importlib
 import pyatspi
 
 from . import braille_generator
@@ -351,8 +353,8 @@ class Script:
                 for package in _settingsManager.settingsPackages:
                     name = package + "." + name
                     try:
-                        module = __import__(name, globals(), locals(), [''])
-                        reload(module)
+                        module = importlib.import_module(name)
+                        imp.reload(module)
                         scriptSettings = module.orca.settings
                     except:
                         pass
@@ -438,7 +440,7 @@ class Script:
         focus    = ["focus:", "object:state-changed:focused"]
         typing   = ["object:text-changed:insert", "object:text-changed:delete"]
         arrowing = ["object:text-caret-moved", "object:text-selection-changed",
-                    "object:selection-changed"]
+                    "object:selection-changed", "object:active-descendant-changed"]
 
         skip = False
         if (event.type in arrowing or event.type in typing) \
