@@ -24,14 +24,11 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2011. Orca Team."
 __license__   = "LGPL"
 
-import debug
-import orca
-import orca_state
+import importlib
 
-from scripts import apps, toolkits
-
-_settingsManager = getattr(orca, '_settingsManager')
-_eventManager = getattr(orca, '_eventManager')
+from . import debug
+from . import orca_state
+from .scripts import apps, toolkits
 
 class ScriptManager:
 
@@ -44,9 +41,9 @@ class ScriptManager:
         self._defaultScript = None
         self._scriptPackages = \
             ["orca-scripts",
-             "scripts",
-             "scripts.apps",
-             "scripts.toolkits"]
+             "orca.scripts",
+             "orca.scripts.apps",
+             "orca.scripts.toolkits"]
         self._appNames = \
             {'Bon Echo':         'Mozilla',
              'Deer Park':        'Mozilla',
@@ -64,7 +61,8 @@ class ScriptManager:
              'empathy-chat':     'empathy',
              'gnome-calculator': 'gcalctool',
              'Nereid':           'Banshee',
-             'vte':              'gnome-terminal'}
+             'vte':              'gnome-terminal',
+             'gnome-terminal-server': 'gnome-terminal'}
         self._toolkitNames = \
             {'gtk':              'GAIL',
              'clutter':          'CALLY'}
@@ -97,7 +95,7 @@ class ScriptManager:
             appAndNameExist = app != None and app.name != ''
         except (LookupError, RuntimeError):
             appAndNameExist = False
-            debug.println(debug.LEVEL_SEVERE,
+            debug.println(debug.LEVEL_FINEST,
                           "getModuleName: %s no longer exists" % app)
 
         if not appAndNameExist:
@@ -106,14 +104,14 @@ class ScriptManager:
         # Many python apps have an accessible name which ends in '.py'.
         # Sometimes OOo has 'soffice.bin' as its name.
         name = app.name.split('.')[0]
-        altNames = self._appNames.keys()
+        altNames = list(self._appNames.keys())
 
-        names = filter(lambda n: n.lower() == name.lower(), altNames)
+        names = [n for n in altNames if n.lower() == name.lower()]
         if names:
             name = self._appNames.get(names[0])
         else:
             for nameList in (self._appModules, self._toolkitModules):
-                names = filter(lambda n: n.lower() == name.lower(), nameList)
+                names = [n for n in nameList if n.lower() == name.lower()]
                 if names:
                     name = names[0]
                     break
@@ -130,7 +128,7 @@ class ScriptManager:
             try:
                 attributes = obj.getAttributes()
             except (LookupError, RuntimeError):
-                debug.println(debug.LEVEL_SEVERE,
+                debug.println(debug.LEVEL_FINEST,
                               "_toolkitForObject: %s no longer exists" % obj)
             else:
                 attrs = dict([attr.split(':', 1) for attr in attributes])
@@ -145,8 +143,8 @@ class ScriptManager:
         if not (app and name):
             return None
 
-        altNames = self._toolkitNames.keys()
-        names = filter(lambda n: n.lower() == name.lower(), altNames)
+        altNames = list(self._toolkitNames.keys())
+        names = [n for n in altNames if n.lower() == name.lower()]
         if names:
             newName = self._toolkitNames.get(names[0])
             debug.println(debug.LEVEL_FINEST,
@@ -158,11 +156,13 @@ class ScriptManager:
             moduleName = '.'.join((package, name))
             debug.println(debug.LEVEL_FINE, "Looking for %s.py" % moduleName)
             try:
-                module = __import__(moduleName, globals(), locals(), [''])
+                module = importlib.import_module(moduleName)
             except ImportError:
                 debug.println(
                     debug.LEVEL_FINE, "Could not import %s.py" % moduleName)
                 continue
+            except OSError:
+                debug.examineProcesses()
 
             debug.println(debug.LEVEL_FINE, "Found %s.py" % moduleName)
             try:
@@ -173,9 +173,9 @@ class ScriptManager:
                 debug.println(debug.LEVEL_FINE, "Loaded %s.py" % moduleName)
                 break
             except:
-                debug.printException(debug.LEVEL_SEVERE)
+                debug.printException(debug.LEVEL_FINEST)
                 debug.println(
-                    debug.LEVEL_SEVERE, "Could not load %s.py" % moduleName)
+                    debug.LEVEL_FINEST, "Could not load %s.py" % moduleName)
 
         return script
 
@@ -211,9 +211,9 @@ class ScriptManager:
         if not app and self._defaultScript:
             return self._defaultScript
 
-        import scripts.default as default
+        from .scripts import default
         script = default.Script(app)
-        _eventManager.registerScriptListeners(script)
+        script.registerEventListeners()
 
         if not app:
             self._defaultScript = script
@@ -240,17 +240,23 @@ class ScriptManager:
             if not toolkitScript:
                 toolkitScript = self._createScript(app, obj)
                 toolkitScripts[objToolkit] = toolkitScript
-                _eventManager.registerScriptListeners(toolkitScript)
+                toolkitScript.registerEventListeners()
             self.toolkitScripts[app] = toolkitScripts
 
-        if not app:
+        try:
+            if not app:
+                appScript = self.getDefaultScript()
+            elif app in self.appScripts:
+                appScript = self.appScripts[app]
+            else:
+                appScript = self._createScript(app, None)
+                self.appScripts[app] = appScript
+                appScript.registerEventListeners()
+        except:
+            msg = "WARNING: Exception getting app script."
+            debug.printException(debug.LEVEL_ALL)
+            debug.println(debug.LEVEL_WARNING, msg)
             appScript = self.getDefaultScript()
-        elif app in self.appScripts:
-            appScript = self.appScripts[app]
-        else:
-            appScript = self._createScript(app, None)
-            self.appScripts[app] = appScript
-            _eventManager.registerScriptListeners(appScript)
 
         # Only defer to the toolkit script for this object if the app script
         # is based on a different toolkit.
@@ -277,7 +283,6 @@ class ScriptManager:
         if not newScript:
             return
 
-        _settingsManager.loadAppSettings(newScript)
         newScript.activate()
         debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: %s (reason=%s)" \
                           % (newScript.name, reason))
@@ -295,11 +300,11 @@ class ScriptManager:
             debug.printException(debug.LEVEL_FINEST)
             return
 
-        appList = self.appScripts.keys()
-        appList = filter(lambda a: a!= None and a not in desktop, appList)
+        appList = list(self.appScripts.keys())
+        appList = [a for a in appList if a != None and a not in desktop]
         for app in appList:
             appScript = self.appScripts.pop(app)
-            _eventManager.deregisterScriptListeners(appScript)
+            appScript.deregisterEventListeners()
             del appScript
 
             try:
@@ -307,8 +312,13 @@ class ScriptManager:
             except KeyError:
                 pass
             else:
-                for toolkitScript in toolkitScripts.values():
-                    _eventManager.deregisterScriptListeners(toolkitScript)
+                for toolkitScript in list(toolkitScripts.values()):
+                    toolkitScript.deregisterEventListeners()
                     del toolkitScript
 
             del app
+
+_manager = ScriptManager()
+
+def getManager():
+    return _manager
