@@ -3,11 +3,13 @@ __all__ = ['Banshee',
            'Eclipse',
            'ekiga',
            'empathy',
+           'evince',
            'evolution',
            'gajim',
            'gcalctool',
            'gdmlogin',
            'gedit',
+           'gnome-documents',
            'gnome-keyring-ask',
            'gnome-mud',
            'gnome-panel',
