@@ -26,19 +26,17 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
-import urlparse, urllib2
+import urllib.parse, urllib.request, urllib.error, urllib.parse
 from gi.repository import Atspi, Atk
 
-import debug
-import generator
-import orca
-import settings
-import sound
-import text_attribute_names
-
-from orca_i18n import _         # for gettext support
-from orca_i18n import ngettext  # for ngettext support
-from orca_i18n import C_        # to provide qualified translatable strings
+from . import debug
+from . import generator
+from . import messages
+from . import object_properties
+from . import settings
+from . import settings_manager
+from . import sound
+from . import text_attribute_names
 
 class Pause:
     """A dummy class to indicate we want to insert a pause into an
@@ -73,15 +71,15 @@ STATE          = "state" # Candidate for sound
 VALUE          = "value" # Candidate for sound
 
 voiceType = {
-    DEFAULT     : settings.DEFAULT_VOICE,
-    UPPERCASE   : settings.UPPERCASE_VOICE,
-    HYPERLINK   : settings.HYPERLINK_VOICE,
-    SYSTEM      : settings.SYSTEM_VOICE,
-    STATE       : settings.SYSTEM_VOICE, # Users may prefer DEFAULT_VOICE here
-    VALUE       : settings.SYSTEM_VOICE, # Users may prefer DEFAULT_VOICE here
+    DEFAULT: settings.DEFAULT_VOICE,
+    UPPERCASE: settings.UPPERCASE_VOICE,
+    HYPERLINK: settings.HYPERLINK_VOICE,
+    SYSTEM: settings.SYSTEM_VOICE,
+    STATE: settings.SYSTEM_VOICE, # Users may prefer DEFAULT_VOICE here
+    VALUE: settings.SYSTEM_VOICE, # Users may prefer DEFAULT_VOICE here
 }
 
-_settingsManager = getattr(orca, '_settingsManager')
+_settingsManager = settings_manager.getManager()
 
 class SpeechGenerator(generator.Generator):
     """Takes accessible objects and produces a string to speak for
@@ -111,7 +109,7 @@ class SpeechGenerator(generator.Generator):
         try:
             soundBite = sound.Sound(sounds[key])
         except:
-            if isinstance(key, basestring):
+            if isinstance(key, str):
                 soundBite = sound.Sound(key)
             else:
                 soundBite = None
@@ -258,7 +256,18 @@ class SpeechGenerator(generator.Generator):
         result = []
         acss = self.voice(SYSTEM)
         role = args.get('role', obj.getRole())
-        if (role != pyatspi.ROLE_UNKNOWN):
+
+        doNotPresent = [pyatspi.ROLE_UNKNOWN]
+
+        # egg-list-box, e.g. privacy panel in gnome-control-center
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            doNotPresent.append(obj.getRole())
+
+        if _settingsManager.getSetting('speechVerbosityLevel') \
+                == settings.VERBOSITY_LEVEL_BRIEF:
+            doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
+
+        if role not in doNotPresent:
             result.append(self.getLocalizedRoleName(obj, role))
             result.extend(acss)
         return result
@@ -462,21 +471,14 @@ class SpeechGenerator(generator.Generator):
             result.extend(self._generateRoleName(obj))
             result.append(self._script.utilities.displayedText(obj))
         else:
-            link_uri_info = urlparse.urlparse(link_uri)
+            link_uri_info = urllib.parse.urlparse(link_uri)
             if link_uri_info[0] in ["ftp", "ftps", "file"]:
                 fileName = link_uri_info[2].split('/')
-                # Translators: this refers to a link to a file, where
-                # the first item is the protocol (ftp, ftps, or file)
-                # and the second item the name of the file being linked
-                # to.
-                #
-                result.append(_("%(uri)s link to %(file)s") \
+                result.append(messages.LINK_TO_FILE \
                               % {"uri" : link_uri_info[0],
                                  "file" : fileName[-1]})
             else:
-                # Translators: this is the protocol of a link eg. http, mailto.
-                #
-                linkOutput = _("%s link") % link_uri_info[0]
+                linkOutput = messages.LINK_WITH_PROTOCOL % link_uri_info[0]
                 text = self._script.utilities.displayedText(obj)
                 if not text:
                     # If there's no text for the link, expose part of the
@@ -501,43 +503,29 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(HYPERLINK)
         link_uri = self._script.utilities.uri(obj)
         if link_uri:
-            link_uri_info = urlparse.urlparse(link_uri)
+            link_uri_info = urllib.parse.urlparse(link_uri)
         else:
             return result
         doc_uri = self._script.utilities.documentFrameURI()
         if doc_uri:
-            doc_uri_info = urlparse.urlparse(doc_uri)
+            doc_uri_info = urllib.parse.urlparse(doc_uri)
             if link_uri_info[1] == doc_uri_info[1]:
                 if link_uri_info[2] == doc_uri_info[2]:
-                    # Translators: this is an indication that a given
-                    # link points to an object that is on the same page.
-                    #
-                    result.append(_("same page"))
+                    result.append(messages.LINK_SAME_PAGE)
                 else:
-                    # Translators: this is an indication that a given
-                    # link points to an object that is at the same site
-                    # (but not on the same page as the link).
-                    #
-                    result.append(_("same site"))
+                    result.append(messages.LINK_SAME_SITE)
             else:
                 # check for different machine name on same site
                 #
                 linkdomain = link_uri_info[1].split('.')
                 docdomain = doc_uri_info[1].split('.')
-                if len(linkdomain) > 1 and docdomain > 1  \
+                if len(linkdomain) > 1 and len(docdomain) > 1  \
                     and linkdomain[-1] == docdomain[-1]  \
                     and linkdomain[-2] == docdomain[-2]:
-                    # Translators: this is an indication that a given
-                    # link points to an object that is at the same site
-                    # (but not on the same page) as the link.
-                    #
-                    result.append(_("same site"))
+                    result.append(messages.LINK_SAME_SITE)
                 else:
-                    # Translators: this is an indication that a given
-                    # link points to an object that is at a different
-                    # site than that of the link.
-                    #
-                    result.append(_("different site"))
+                    result.append(messages.LINK_DIFFERENT_SITE)
+
         if result:
             result.extend(acss)
         return result
@@ -555,27 +543,21 @@ class SpeechGenerator(generator.Generator):
         if not uri:
             return result
         try:
-            x = urllib2.urlopen(uri)
+            x = urllib.request.urlopen(uri)
             try:
                 sizeString = x.info()['Content-length']
             except KeyError:
                 pass
-        except (ValueError, urllib2.URLError, OSError):
+        except (ValueError, urllib.error.URLError, OSError):
             pass
         if sizeString:
             size = int(sizeString)
             if size < 10000:
-                # Translators: This is the size of a file in bytes
-                #
-                result.append(ngettext("%d byte", "%d bytes", size) % size)
+                result.append(messages.fileSizeBytes(size))
             elif size < 1000000:
-                # Translators: This is the size of a file in kilobytes
-                #
-                result.append(_("%.2f kilobytes") % (float(size) * .001))
+                result.append(messages.FILE_SIZE_KB % (float(size) * .001))
             elif size >= 1000000:
-                # Translators: This is the size of a file in megabytes
-                #
-                result.append(_("%.2f megabytes") % (float(size) * .000001))
+                result.append(messages.FILE_SIZE_MB % (float(size) * .000001))
         if result:
             result.extend(acss)
         return result
@@ -721,10 +703,7 @@ class SpeechGenerator(generator.Generator):
         self._restoreRole(oldRole, args)
         if not result and _settingsManager.getSetting('speakBlankLines') \
            and not args.get('readingRow', False):
-            # Translators: "blank" is a short word to mean the
-            # user has navigated to an empty line.
-            #
-            result.append(_("blank"))
+            result.append(messages.BLANK)
             if result:
                 result.extend(acss)
 
@@ -774,10 +753,7 @@ class SpeechGenerator(generator.Generator):
 
         if checkIfSelected \
            and state and not state.contains(pyatspi.STATE_SELECTED):
-            # Translators: this is in reference to a table cell being
-            # selected or not.
-            #
-            result.append(C_("tablecell", "not selected"))
+            result.append(object_properties.STATE_UNSELECTED_TABLE_CELL)
             result.extend(acss)
 
         return result
@@ -804,9 +780,7 @@ class SpeechGenerator(generator.Generator):
             index = self._script.utilities.cellIndex(obj)
             col = table.getColumnAtIndex(index)
         if col >= 0:
-            # Translators: this is in references to a column in a
-            # table.
-            result.append(_("column %d") % (col + 1))
+            result.append(messages.TABLE_COLUMN % (col + 1))
         if result:
             result.extend(acss)
         return result
@@ -833,9 +807,7 @@ class SpeechGenerator(generator.Generator):
             index = self._script.utilities.cellIndex(obj)
             row = table.getRowAtIndex(index)
         if row >= 0:
-            # Translators: this is in references to a row in a table.
-            #
-            result.append(_("row %d") % (row + 1))
+            result.append(messages.TABLE_ROW % (row + 1))
         if result:
             result.extend(acss)
         return result
@@ -862,14 +834,10 @@ class SpeechGenerator(generator.Generator):
             index = self._script.utilities.cellIndex(obj)
             col = table.getColumnAtIndex(index)
             row = table.getRowAtIndex(index)
-            # Translators: this is in references to a column in a
-            # table.
-            result.append(_("column %(index)d of %(total)d") \
+            result.append(messages.TABLE_COLUMN_DETAILED \
                           % {"index" : (col + 1),
                              "total" : table.nColumns})
-            # Translators: this is in reference to a row in a table.
-            #
-            result.append(_("row %(index)d of %(total)d") \
+            result.append(messages.TABLE_ROW_DETAILED \
                           % {"index" : (row + 1),
                              "total" : table.nRows})
         if result:
@@ -902,10 +870,7 @@ class SpeechGenerator(generator.Generator):
                 row = table.getRowAtIndex(index)
                 col = table.getColumnAtIndex(index)
                 if row + 1 == table.nRows and col + 1 == table.nColumns:
-                    # Translators: This is to indicate to the user that
-                    # he/she is in the last cell of a table in a document.
-                    #
-                    result.append(_("End of table"))
+                    result.append(messages.TABLE_END)
         if result:
             result.extend(acss)
         return result
@@ -990,18 +955,13 @@ class SpeechGenerator(generator.Generator):
                         #
                         if key == "weight":
                             if int(attribute) > 400:
-                                attribStr += " "
-                                # Translators: bold as in the font sense.
-                                #
-                                attribStr += _("bold")
+                                attribStr += " %s" % messages.BOLD
                         elif key == "underline":
                             if attribute != "none":
-                                attribStr += " "
-                                attribStr += localizedKey
+                                attribStr += " %s" % localizedKey
                         elif key == "style":
                             if attribute != "normal":
-                                attribStr += " "
-                                attribStr += localizedValue
+                                attribStr += " %s" % localizedValue
                         else:
                             attribStr += " "
                             attribStr += (localizedKey + " " + localizedValue)
@@ -1009,11 +969,7 @@ class SpeechGenerator(generator.Generator):
             # Also check to see if this is a hypertext link.
             #
             if self._script.utilities.linkIndex(obj, textOffset) >= 0:
-                attribStr += " "
-                # Translators: this indicates that this piece of
-                # text is a hypertext link.
-                #
-                attribStr += _("link")
+                attribStr += " %s" % messages.LINK
 
         return attribStr
 
@@ -1063,8 +1019,7 @@ class SpeechGenerator(generator.Generator):
                 # characters). Until we do, this expansion is better than
                 # presenting the actual embedded object character.
                 #
-                unicodeText = line.decode("UTF-8")
-                if self._script.EMBEDDED_OBJECT_CHARACTER in unicodeText:
+                if self._script.EMBEDDED_OBJECT_CHARACTER in line:
                     line = self._script.utilities.expandEOCs(
                         obj, startOffset, endOffset)
                 line = self._script.utilities.adjustForRepeats(line)
@@ -1074,10 +1029,7 @@ class SpeechGenerator(generator.Generator):
                     pyatspi.TEXT_BOUNDARY_CHAR)
                 if char[0] == "\n" and startOffset == caretOffset \
                        and _settingsManager.getSetting('speakBlankLines'):
-                    # Translators: "blank" is a short word to mean the
-                    # user has navigated to an empty line.
-                    #
-                    textContents = (_("blank"))
+                    textContents = (messages.BLANK)
 
         self._script.generatorCache['textInformation'] = \
             [textContents, startOffset, endOffset, selected]
@@ -1170,11 +1122,7 @@ class SpeechGenerator(generator.Generator):
             self._getTextInformation(obj)
 
         if selected:
-            # Translators: when the user selects (highlights) text in
-            # a document, Orca lets them know this.
-            #
-            text = C_("text", "selected")
-            result.append(text)
+            result.append(messages.TEXT_SELECTED)
             result.extend(acss)
         return result
 
@@ -1199,10 +1147,7 @@ class SpeechGenerator(generator.Generator):
                 [string, startOffset, endOffset] = \
                    textObj.getTextAtOffset(0, pyatspi.TEXT_BOUNDARY_LINE_START)
                 if startOffset == 0 and endOffset == len(string):
-                    # Translators: when the user selects (highlights) text in
-                    # a document, Orca lets them know this.
-                    #
-                    result = [C_("text", "selected")]
+                    result = [messages.TEXT_SELECTED]
                     result.extend(acss)
         return result
 
@@ -1227,13 +1172,10 @@ class SpeechGenerator(generator.Generator):
         if not line:
             [line, caretOffset, startOffset] = \
               self._script.getTextLineAtCaret(obj)
-        # For the purpose of speaking the text indentation, replace
-        # occurances of UTF-8 '\302\240' (non breaking space) with
-        # spaces.
-        #
-        line = line.replace("\302\240",  " ")
-        line = line.decode("UTF-8")
 
+        # For the purpose of speaking the text indentation, replace
+        # occurances the non breaking space character with spaces.
+        line = line.replace("\u00a0", " ")
         spaceCount = 0
         tabCount = 0
         utterance = ""
@@ -1243,24 +1185,12 @@ class SpeechGenerator(generator.Generator):
                 spaceCount += 1
                 offset += 1
             if spaceCount:
-                # Translators: this is the number of space characters on a line
-                # of text.
-                #
-                utterance += ngettext("%d space",
-                                      "%d spaces",
-                                      spaceCount) % spaceCount + " "
-
+                utterance += "%s " % messages.spacesCount(spaceCount)
             while (offset < len(line)) and line[offset] == '\t':
                 tabCount += 1
                 offset += 1
             if tabCount:
-                # Translators: this is the number of tab characters on a line
-                # of text.
-                #
-                utterance += ngettext("%d tab",
-                                      "%d tabs",
-                                      tabCount) % tabCount + " "
-
+                utterance += "%s " % messages.tabsCount(tabCount)
             if not (spaceCount  or tabCount):
                 break
             spaceCount  = tabCount = 0
@@ -1323,12 +1253,7 @@ class SpeechGenerator(generator.Generator):
                 (value.currentValue
                  / (value.maximumValue - value.minimumValue)) \
                 * 100.0
-            # Translators: this is the percentage value of a progress bar.
-            #
-            percentage = ngettext("%d percent",
-                                  "%d percent",
-                                  percentValue) % percentValue
-            result.append(percentage)
+            result.append(messages.percentage(percentValue))
         if result:
             result.extend(acss)
         return result
@@ -1393,11 +1318,7 @@ class SpeechGenerator(generator.Generator):
         childNodes = self._script.utilities.childNodes(obj)
         children = len(childNodes)
         if children:
-            # Translators: this is the number of items in a layered
-            # pane or table.
-            #
-            itemString = ngettext("%d item", "%d items", children) % children
-            result.append(itemString)
+            result.append(messages.itemCount(children))
             result.extend(acss)
         return result
 
@@ -1421,10 +1342,7 @@ class SpeechGenerator(generator.Generator):
                 hasItems = True
                 break
         if not hasItems:
-            # Translators: this is the number of items in a layered pane
-            # or table.
-            #
-            result.append(_("0 items"))
+            result.append(messages.ZERO_ITEMS)
             result.extend(acss)
         return result
 
@@ -1442,13 +1360,33 @@ class SpeechGenerator(generator.Generator):
         result = []
         acss = self.voice(SYSTEM)
         if not obj.childCount:
-            # Translators: this is the number of items in a layered pane
-            # or table.
-            #
-            result.append(_("0 items"))
+            result.append(messages.ZERO_ITEMS)
             result.extend(acss)
         return result
 
+    def _generateFocusedItem(self, obj, **args):
+        result = []
+        role = args.get('role', obj.getRole())
+        if role != pyatspi.ROLE_LIST:
+            return result
+
+        try:
+            s = obj.querySelection()
+        except NotImplementedError:
+            isFocused = \
+                lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
+            items = pyatspi.utils.findAllDescendants(obj, isFocused)
+        else:
+            items = [s.getSelectedChild(i) for i in range(s.nSelectedChildren)]
+            if not items and obj.childCount:
+                items.append(obj[0])
+
+        items = list(map(self._generateName, items))
+        for item in items:
+            result.extend(item)
+
+        return result
+
     def _generateSelectedItemCount(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) indicating how many items are selected in this
@@ -1463,7 +1401,8 @@ class SpeechGenerator(generator.Generator):
         # TODO - JD: Is there a better way to do this other than
         # hard-coding it in?
         #
-        if args.get('role', obj.getRole()) == pyatspi.ROLE_ICON:
+        if args.get('role', obj.getRole()) \
+                in [pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS]:
             obj = obj.parent
         childCount = obj.childCount
         selectedItems = []
@@ -1476,15 +1415,7 @@ class SpeechGenerator(generator.Generator):
                 selectedItems.append(child)
             if state.contains(pyatspi.STATE_FOCUSED):
                 currentItem = child.getIndexInParent() + 1
-        # Translators: this is a count of the number of selected icons
-        # and the count of the total number of icons within an icon panel.
-        # An example of an icon panel is the Nautilus folder view.
-        #
-        countString = ngettext("%(index)d of %(total)d item selected",
-                               "%(index)d of %(total)d items selected",
-                               childCount) \
-                      % {"index" : totalSelectedItems,
-                         "total" : childCount}
+        countString = messages.selectedItemsCount(totalSelectedItems, childCount)
         result.append(countString)
         result.extend(acss)
         result.append(self._script.formatting.getString(
@@ -1533,12 +1464,7 @@ class SpeechGenerator(generator.Generator):
         except:
             alertAndDialogCount = 0
         if alertAndDialogCount > 0:
-            # Translators: this tells the user how many unfocused
-            # alert and dialog windows that this application has.
-            #
-            result.append(ngettext("%d unfocused dialog",
-                            "%d unfocused dialogs",
-                            alertAndDialogCount) % alertAndDialogCount)
+            result.append(messages.dialogCountSpeech(alertAndDialogCount))
             result.extend(acss)
         return result
 
@@ -1622,10 +1548,7 @@ class SpeechGenerator(generator.Generator):
         if args.get('role', obj.getRole()) == pyatspi.ROLE_ICON \
            and args.get('formatType', None) \
                in ['basicWhereAmI', 'detailedWhereAmI']:
-            # Translators: this is an alternative name for the
-            # parent object of a series of icons.
-            #
-            return [_("Icon panel")]
+            return [object_properties.ROLE_ICON_PANEL]
         elif obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
             obj = obj.parent
         return self._generateRoleName(obj.parent)
@@ -1780,11 +1703,7 @@ class SpeechGenerator(generator.Generator):
         if button and button.getState().contains(pyatspi.STATE_SENSITIVE):
             name = self._generateName(button)
             if name:
-                # Translators: The "default" button in a dialog box is the
-                # button that gets activated when Enter is pressed anywhere
-                # within that dialog box.
-                #
-                result.append(_("Default button is %s") % name[0])
+                result.append(messages.DEFAULT_BUTTON_IS % name[0])
         return result
 
     def generateDefaultButton(self, obj, **args):
@@ -1833,13 +1752,7 @@ class SpeechGenerator(generator.Generator):
         alertAndDialogCount = \
                     self._script.utilities.unfocusedAlertAndDialogCount(obj)
         if alertAndDialogCount > 0:
-            # Translators: this tells the user how many unfocused
-            # alert and dialog windows that this application has.
-            #
-            dialogs = []
-            dialogs.append(ngettext("%d unfocused dialog",
-                                    "%d unfocused dialogs",
-                                    alertAndDialogCount) % alertAndDialogCount)
+            dialogs = [messages.dialogCountSpeech(alertAndDialogCount)]
             dialogs.extend(acss)
             result.append(dialogs)
         return result
