@@ -26,10 +26,7 @@
 #
 # pylint: disable-msg=W0142
 
-"""Provides an Orca speech server for Speech Dispatcher backend.
-
-NOTE: THIS IS EXPERIMENTAL ONLY AND IS NOT A SUPPORTED COMPONENT OF ORCA.
-"""
+"""Provides an Orca speech server for Speech Dispatcher backend."""
 
 __id__ = "$Id$"
 __version__   = "$Revision$"
@@ -38,17 +35,18 @@ __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006-2008 Brailcom, o.p.s."
 __license__   = "LGPL"
 
-from gi.repository import GObject
+from gi.repository import GLib
 import re
 
 from . import chnames
 from . import debug
+from . import guilabels
+from . import messages
 from . import speechserver
 from . import settings
 from . import orca_state
 from . import punctuation_settings
 from .acss import ACSS
-from .orca_i18n import _
 
 try:
     import speechd
@@ -72,14 +70,7 @@ class SpeechServer(speechserver.SpeechServer):
     _active_servers = {}
     
     DEFAULT_SERVER_ID = 'default'
-    
-    # Translators: "Default Synthesizer" will appear in the list of available
-    # speech engines as a special item.  It refers to the default engine
-    # configured within the speech subsystem.  Apart from this item, the user
-    # will have a chance to select a particular speech engine by its real
-    # name, such as Festival, IBMTTS, etc.
-    #
-    _SERVER_NAMES = {DEFAULT_SERVER_ID: _("Default Synthesizer")}
+    _SERVER_NAMES = {DEFAULT_SERVER_ID: guilabels.DEFAULT_SYNTHESIZER}
     
     KEY_NAMES = {
         '_':     'underscore',
@@ -88,12 +79,8 @@ class SpeechServer(speechserver.SpeechServer):
         '"':     'double-quote',
         }
 
-
     def getFactoryName():
-        # Translators: this is the name of a speech synthesis system
-        # called "Speech Dispatcher".
-        #
-        return _("Speech Dispatcher")
+        return guilabels.SPEECH_DISPATCHER
     getFactoryName = staticmethod(getFactoryName)
 
     def getSpeechServers():
@@ -168,16 +155,8 @@ class SpeechServer(speechserver.SpeechServer):
             speechd.CallbackType.END: speechserver.SayAllContext.COMPLETED,
            #speechd.CallbackType.INDEX_MARK:speechserver.SayAllContext.PROGRESS,
             }
-        # Translators: This string will appear in the list of
-        # available voices for the current speech engine.  %s will be
-        # replaced by the name of the current speech engine, such as
-        # "Festival default voice" or "IBMTTS default voice".  It
-        # refers to the default voice configured for given speech
-        # engine within the speech subsystem.  Apart from this item,
-        # the list will contain the names of all available "real"
-        # voices provided by the speech engine.
-        #
-        self._default_voice_name = _("%s default voice") % serverId
+
+        self._default_voice_name = guilabels.SPEECH_DEFAULT_VOICE % serverId
         
         try:
             self._init()
@@ -197,6 +176,10 @@ class SpeechServer(speechserver.SpeechServer):
         mode = self._PUNCTUATION_MODE_MAP[settings.verbalizePunctuationStyle]
         client.set_punctuation(mode)
 
+    def updateCapitalizationStyle(self):
+        """Updates the capitalization style used by the speech server."""
+        self._client.set_cap_let_recogn(settings.capitalizationStyle)
+
     def updatePunctuationLevel(self):
         """ Punctuation level changed, inform this speechServer. """
         mode = self._PUNCTUATION_MODE_MAP[settings.verbalizePunctuationStyle]
@@ -257,7 +240,10 @@ class SpeechServer(speechserver.SpeechServer):
                 if current.get(acss_property) != value:
                     method(value)
                     current[acss_property] = value
-            elif acss_property == ACSS.FAMILY and current.get(acss_property) \
+            elif acss_property == ACSS.AVERAGE_PITCH:
+                method(5.0)
+                current[acss_property] = 5.0
+            elif acss_property == ACSS.FAMILY \
                     and acss == settings.voices[settings.DEFAULT_VOICE]:
                 # We need to explicitly reset (at least) the family.
                 # See bgo#626072.
@@ -278,13 +264,7 @@ class SpeechServer(speechserver.SpeechServer):
         Returns a text string with the punctuation symbols adjusted accordingly.
         """
 
-        # Translators: we replace the ellipses (both manual and UTF-8)
-        # with a spoken string.  The extra space you see at the beginning
-        # is because we need the speech synthesis engine to speak the
-        # new string well.  For example, "Open..." turns into
-        # "Open dot dot dot".
-        #
-        spokenEllipsis = _(" dot dot dot") + " "
+        spokenEllipsis = messages.SPOKEN_ELLIPSIS + " "
         newText = re.sub(ELLIPSIS, spokenEllipsis, oldText)
         symbols = set(re.findall(PUNCTUATION, newText))
         for symbol in symbols:
@@ -352,9 +332,9 @@ class SpeechServer(speechserver.SpeechServer):
                         context.currentOffset = context.startOffset
                 elif t == speechserver.SayAllContext.COMPLETED:
                     context.currentOffset = context.endOffset
-                GObject.idle_add(orca_callback, context, t)
+                GLib.idle_add(orca_callback, context, t)
                 if t == speechserver.SayAllContext.COMPLETED:
-                    GObject.idle_add(self._say_all, iterator, orca_callback)
+                    GLib.idle_add(self._say_all, iterator, orca_callback)
             self._speak(context.utterance, acss, callback=callback,
                         event_types=list(self._CALLBACK_TYPE_MAP.keys()))
         return False # to indicate, that we don't want to be called again.
@@ -372,8 +352,9 @@ class SpeechServer(speechserver.SpeechServer):
         acss[ACSS.RATE] = max(0, min(99, rate + delta))
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech rate is now %d" % rate)
-        # Translators: This string announces speech rate change.
-        self.speak(decrease and _("slower.") or _("faster."), acss=acss)
+
+        self.speak(decrease and messages.SPEECH_SLOWER \
+                   or messages.SPEECH_FASTER, acss=acss)
 
     def _change_default_speech_pitch(self, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
@@ -385,8 +366,9 @@ class SpeechServer(speechserver.SpeechServer):
         acss[ACSS.AVERAGE_PITCH] = max(0, min(9, pitch + delta))
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech pitch is now %d" % pitch)
-        # Translators: This string announces speech pitch change.
-        self.speak(decrease and _("lower.") or _("higher."), acss=acss)
+
+        self.speak(decrease and messages.SPEECH_LOWER \
+                   or messages.SPEECH_HIGHER, acss=acss)
 
     def getInfo(self):
         return [self._SERVER_NAMES.get(self._id, self._id), self._id]
@@ -398,8 +380,9 @@ class SpeechServer(speechserver.SpeechServer):
         locale = getlocale(LC_MESSAGES)[0]
         if locale is None or locale == 'C':
             lang = None
+            dialect = None
         else:
-            lang = locale.split('_')[0]
+            lang, dialect = locale.split('_')
         voices = ((self._default_voice_name, lang, None),)
         try:
             # This command is not available with older SD versions.
@@ -411,6 +394,7 @@ class SpeechServer(speechserver.SpeechServer):
         families = [speechserver.VoiceFamily({ \
               speechserver.VoiceFamily.NAME: name,
               #speechserver.VoiceFamily.GENDER: speechserver.VoiceFamily.MALE,
+              speechserver.VoiceFamily.DIALECT: dialect,
               speechserver.VoiceFamily.LOCALE: lang})
                     for name, lang, dialect in voices]
         return families
@@ -434,7 +418,7 @@ class SpeechServer(speechserver.SpeechServer):
                 self._speak(utterance, acss)
 
     def sayAll(self, utteranceIterator, progressCallback):
-        GObject.idle_add(self._say_all, utteranceIterator, progressCallback)
+        GLib.idle_add(self._say_all, utteranceIterator, progressCallback)
 
     def speakCharacter(self, character, acss=None):
         self._apply_acss(acss)
